/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.servlets.keyretrieval;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.openexchange.guard.common.servlets.utils.AntiAbuseUtils;
import com.openexchange.guard.common.servlets.utils.ServletUtils;
import com.openexchange.guard.keymanagement.commons.antiabuse.AntiAbuseAction;
import com.openexchange.guard.keymanagement.commons.antiabuse.AntiAbuseWrapper;
import com.openexchange.guard.servlets.GuardServletAction;
import com.openexchange.guard.servlets.authentication.AuthorizationHeaderParser;
import com.openexchange.guard.servlets.authentication.GuardUserSession;


/**
 * {@link RetrievalAllExposedKeysAction}
 *
 * Allows retrieving deleted keys for a user, which have been exposed
 * using the support API ({@link com.openexchange.guard.servlets.supportApi.ExposeKeyAction})
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v2.10.0
 */
public class RetrievalAllExposedKeysAction extends GuardServletAction {

    private static final String BASIC_AUTH_PREFIX = "Basic";
    private static final String CONTENT_TYPE = "application/pgp-keys";
    private static final boolean MANDATORY = true;

    /* (non-Javadoc)
     * @see com.openexchange.guard.servlets.GuardServletAction#doActionInternal(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, com.openexchange.guard.servlets.authentication.GuardUserSession)
     */
    @Override
    protected void doActionInternal(HttpServletRequest request, HttpServletResponse response, GuardUserSession userSession) throws Exception {

        String email = null;
        String password = null;
        final int userId = ServletUtils.getIntParameter(request, "userId", MANDATORY);
        final int contextId = ServletUtils.getIntParameter(request, "contextId", MANDATORY);

        //Get the auth header if available
        String authHeader = new AuthorizationHeaderParser().parse(BASIC_AUTH_PREFIX, request);
        if(authHeader != null) {

            //email & password from auth-header
            if(authHeader != null && authHeader.contains(":")) {
                String[] credentials = authHeader.split(":");
                if (credentials != null && credentials.length >= 2) {
                    email = credentials[0];
                    password = credentials[1];
                }
            }

            //Export
            final String closureEmail = email;
            final String closurePassword = password;
            AntiAbuseWrapper antiAbuseWrapper = getAntiAbuseWrapper(AntiAbuseUtils.getAllowParameters(request, email, password));
            final String exportData = antiAbuseWrapper.doAction(new AntiAbuseAction<String>() {
                @Override
                public String doAction() throws Exception {
                    return new KeyRetriever().exportAllKeys(closureEmail, userId, contextId, closurePassword);
                }

            });

            if(exportData != null) {
                response.setContentLength(exportData.length());
                response.setHeader("Content-Disposition", "attachment; filename=\"export.asc\"");
                ServletUtils.sendOK(response, CONTENT_TYPE, exportData);
            }
            else {
                response.sendError(HttpServletResponse.SC_NOT_FOUND, "No exposed keys found for user.");
            }
        }
        else {
            //Client did not provide credentials
            response.setHeader("WWW-Authenticate", "Basic");
            response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
        }
    }
}
